@@ -1,24 +1,25 @@
+
 package App::Build;
 
-use strict;
-
-# undo @ARGV manipulation done by App::Options, to allow Module::Build to work
-my @orig_ARGV; BEGIN { @orig_ARGV = @ARGV; }
 use App::Options;
-@ARGV = @orig_ARGV;
-
 use Module::Build;
 use Cwd ();
 use File::Spec;
-use File::Basename qw();
-use File::Path qw();
+use Sysadm::Install qw(:all);
 
-our $VERSION = "0.74_04";
-our @ISA = ("Module::Build");
+# until I get to 1.0, I will update the version number manually
+$VERSION = "0.10";
+#$VERSION = do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r};
+
+@ISA = ("Module::Build");
+
+use strict;
+
+delete $ENV{PREFIX};   # Module::Build protests if this var is set
 
 =head1 NAME
 
-App::Build - extends Module::Build to build/install/configure entire applications (i.e. web applications), not just modules and programs
+App::Build - a base/default class for building/installing applications
 
 =head1 SYNOPSIS
 
@@ -27,9 +28,11 @@ App::Build - extends Module::Build to build/install/configure entire application
 
   use App::Build;
 
+  shift(@ARGV) if ($#ARGV > -1 && $ARGV[0] eq "Build");
+
   my $build = App::Build->new (
-      dist_name         => "App-Build-Foo",
-      dist_version      => "1.0",
+      dist_name         => "App-Build",
+      dist_version_from => "lib/App/Build.pm",
       dist_author       => "stephen.adkins\@officevision.com",
       extra_dirs        => [ "htdocs", "cgi-bin", "etc", "var" ],
       license           => "perl",
@@ -42,30 +45,23 @@ App::Build - extends Module::Build to build/install/configure entire application
 
 =head1 DESCRIPTION
 
-App::Build is a subclass of Module::Build, so you can use
-it in place of Module::Build when creating your "Build.PL"
-installation scripts.
-
-Module::Build is good at
-installing perl modules and programs/scripts.  Full applications
-(i.e. web applications) need to install other files such as
-web pages, images, CSS style sheets, javascript files,
-CGI programs, data files, and configuration data.
-App::Build addresses these issues.
-
 The vision of App::Build is to make installing entirely
 functional perl applications (particularly web applications)
-as easy as installing individual modules from CPAN.
+as easy as installing individual models from CPAN.
 
 An ISP customer (or other unprivileged user)
 who has shell access should be able to install any number
 of available applications from CPAN simply by typing the
 usual
 
-  perl -MCPAN -e "install App::Build::Foo"
+  perl -MCPAN -e "install App-Build-Foo"
 
 and the "Foo" application is installed on his account.
 
+The goal is to make installing entire perl applications
+as easy as installing a simple set of modules.
+App::Build is a subclass of Module::Build.
+
 App::Build does this by implementing the following features.
 
 1. Distributions that use App::Build instead of Module::Build
@@ -92,7 +88,7 @@ App-Build-Foo and upload it to CPAN.
 When someone uses the CPAN shell, they can install the
 application simply by typing
 
- install App::Build::Foo
+ install App-Build-Foo
 
 Within the App-Build-Foo distribution would be a module,
 App::Build::Foo, which would be a subclass of App::Build.
@@ -130,35 +126,17 @@ installations.
 
 =head1 App::Build CONFIGURABILITY
 
-Since App::Build uses App::Options, App::Options makes all the
-of the --var=value options available via the global %App::options hash.
-App::Build however does not remove the --var=value options from @ARGV.
+Since App::Build uses App::Options, App::Options strips off all
+of the --var=value options out of @ARGV and makes them available
+via the global %App::options hash.
 
-This will be put to good use sometime in the future.
+At build/install time, Build.PL (using Module::Build) is 
+configurable using VAR=VALUE pairs on the command line.
 
-=head1 FIX-UPS
-
-Module::Build complains if the PREFIX environment variable is
-set.  App::Build doesn't.  It just ignores it.
-
-The CPAN shell (for some reason I don't understand) runs Build.PL
-as "perl Build.PL Build" and this fails.
-App::Build just throws away the "Build" so that the default "build"
-action is invoked.
-
-Module::Build deprecated the PREFIX option to Makefile.PL
-(i.e. "perl Makefile.PL PREFIX=/usr/foo").  App::Build just makes
-the PREFIX option a synonym for "install_base", which does
-essentially the same thing.
+(more on this later)
 
 =cut
 
-delete $ENV{PREFIX};   # Module::Build protests if this var is set
-
-# Enable the continued use of the PREFIX=$PREFIX option
-# (from Makefile.PL and ExtUtils::MakeMaker) by making it
-# an alias for the "install_base" option of Module::Build.
-
 ######################################################################
 # BUILD: enhancements to "perl Build.PL"
 ######################################################################
@@ -183,84 +161,57 @@ distributions.
 
 sub new {
     my ($class, %args) = @_;
+    #print "new($class, {", join(",",%args), "})\n";
+
     my $obj = $class->SUPER::new(%args);
 
-    $obj->_enhance_install_paths() if $obj->_prefix;
-    $obj->_get_supporting_software();
+    my $dist_name = $args{dist_name};
+    if (!$dist_name) {
+        die "must provide a dist_name" if ($dist_name eq "App-Build");
+    }
+
+    $obj->_enhance_install_paths();
+
+#    my $tag = lc($dist_name);
+#    $tag =~ s/^app-build-//;
+#
+#    my $url  = $App::options{"$tag.url"};
+#    ($url) || die "URL [$tag.url] does not exist";
+#
+#    my $file = $App::options{"$tag.file"};
+#    if (!$file) {
+#        $file = $url;
+#        $file =~ s!.*/!!;
+#    }
+#    ($file) || die "File [$tag.file] does not exist";
+#
+#    my $subdir = $App::options{"$tag.subdir"};
+#    if (!$subdir) {
+#        $subdir = $file;
+#        $subdir =~ s!\.tar.gz$!!;
+#        $subdir =~ s!\.tgz$!!;
+#    }
+#    ($subdir) || die "Subdir [$tag.subdir] does not exist";
+#
+#    my $prefix = $App::options{install_prefix} || $App::options{prefix};
+#    my $archive_dir = $App::options{archive_dir} || "archive";
+#    mkdir($archive_dir) if (! -d $archive_dir);
+#
+#    my $archive = "$archive_dir/$file";
+#
+#    (-d $archive_dir) || die "Archive Directory [$archive_dir] does not exist";
+#    (-w $archive_dir) || die "Archive Directory [$archive_dir] not writeable";
+#
+#    $class->mirror($url, $archive);
+#    $class->unpack($archive, "unpack", $subdir);
+
+    #print "new() = $obj\n";
+    #print "obj = {", join(",", %$obj), "}\n";
+    #print "obj{properties} = {", join(",", %{$obj->{properties}}), "}\n";
 
     return($obj);
 }
 
-=head2 read_config()
-
-Overridden to transparently call C<_enhance_install_paths()>.
-
-=cut
-
-sub read_config {
-    my ($self) = @_;
-
-    $self->SUPER::read_config();
-    $self->_enhance_install_paths() if $self->_prefix;
-}
-
-=head2 install_base()
-
-Overridden to transparently call C<_enhance_install_paths()>.
-
-=cut
-
-sub install_base {
-    my ($self, @args) = @_;
-
-    my $ret = $self->SUPER::install_base(@args);
-    $self->_enhance_install_paths() if $self->_prefix;
-
-    return $ret;
-}
-
-=head2 _get_supporting_software()
-
-Downloads supporting software (if necessary), unpacks it, compiles it,
-and installs it.
-
-=cut
-
-sub _get_supporting_software {
-    my ($self) = @_;
-
-    my $tag = $self->_app_tag();
-
-    my $url  = $App::options{"$tag.url"};   # check to see if there was software to download
-    if ($url) {
-        my $file = $App::options{"$tag.file"};
-        if (!$file) {
-            $file = $url;
-            $file =~ s!.*/!!;
-        }
-        ($file) || die "File [$tag.file] does not exist";
-
-        my $subdir = $App::options{"$tag.subdir"};
-        if (!$subdir) {
-            $subdir = $file;
-            $subdir =~ s!\.tar.gz$!!;
-            $subdir =~ s!\.tgz$!!;
-        }
-        ($subdir) || die "Subdir [$tag.subdir] does not exist";
-
-        my $archive_dir = $App::options{archive_dir} || "archive";
-        mkdir($archive_dir) if (! -d $archive_dir);
-
-        my $archive = "$archive_dir/$file";
-
-        (-d $archive_dir) || die "Archive Directory [$archive_dir] does not exist";
-        (-w $archive_dir) || die "Archive Directory [$archive_dir] not writeable";
-
-        $self->mirror($url, $archive);
-        $self->unpack($archive, "unpack", $subdir);
-    }
-}
-
 =head2 _app_tag()
 
 This lowercase-ifies the dist_name, removes "app-build-" from the front,
@@ -346,7 +297,7 @@ the "blib" subdirectory in preparation for installation.
 sub ACTION_code {
     my ($self) = @_;
     $self->SUPER::ACTION_code(); # call this first (creates "blib" dir if necessary)
-    $self->process_app_files();  # NEW. call this to copy "extra_dirs" to "blib"
+    $self->process_app_files();  # NEW. call this to copy "extra_libs" to "blib"
 }
 
 =head2 _added_to_INC()
@@ -386,17 +337,9 @@ are hashrefs of attributes. i.e.
        htdocs => {
            dest_dir => "htdocs",
        },
-       "cgi-bin" => {  # any dir ending in "bin" contains executable scripts
-           dest_dir => "cgi-bin",
-       },
-       support => {
-           dest_dir => "support",
-           executable => 1,  # treat contents as executable scripts
-       },
    },
 
-So far, only the "dest_dir" attribute is defined.  The "dest_dir" attribute
-can be overridden using the "--install_path" parameter.
+So far, only the "dest_dir" attribute is defined.
 
 =cut
 
@@ -448,11 +391,6 @@ sub _get_extra_dirs_attributes {
              @extra_dirs = split(/,/,$properties->{extra_dirs});
              $extra_dirs = { map { $_ => { dest_dir => $_ } } @extra_dirs };
         }
-        foreach my $dir (@extra_dirs) {
-            $extra_dirs->{$dir}{dest_dir} = $self->install_path($dir)
-              if $self->install_path($dir);
-            $extra_dirs->{$dir}{dest_dir} = $dir if (!$extra_dirs->{$dir}{dest_dir});
-        }
     }
     return($extra_dirs);
 }
@@ -470,45 +408,21 @@ sub process_app_files {
     my ($path, $files);
 
     my @extra_dirs = $self->_get_extra_dirs();
-    my $extra_dirs = $self->_get_extra_dirs_attributes();
+    # print "process_app_files(): extra_dirs=[@extra_dirs]\n";
 
     my $blib = $self->blib;
-    my ($contains_executables, $result, $target_file);
     foreach my $dir (@extra_dirs) {
         if (-d $dir) {
-            $contains_executables = $extra_dirs->{$dir}{executable};
-            $contains_executables = ($dir =~ /bin$/) ? 1 : 0 if (!defined $contains_executables);
             $path = File::Spec->catfile($blib, $dir), 
             File::Path::mkpath($path);
             $files = $self->_find_all_files($dir);
-            my ($should_be_executable);
             while (my ($file, $dest) = each %$files) {
-                $target_file = File::Spec->catfile($blib, $dest);
-                $result = $self->copy_if_modified(from => $file, to => $target_file) || "";
-                if ($result && $contains_executables) {
-                    $self->fix_shebang_line($result);
-                    $self->make_executable($result)
-                      if $self->_should_be_executable($result);
-                }
+                $self->copy_if_modified(from => $file, to => File::Spec->catfile($blib, $dest) );
             }
         }
     }
 }
 
-sub _should_be_executable {
-    my ($self, $file) = @_;
-
-    # copied from Module::Build::Base::fix_shebang_line
-    my $FIXIN = IO::File->new($file) or die "Can't process '$file': $!";
-    local $/ = "\n";
-    chomp(my $line = <$FIXIN>);
-    if ($line =~ /^\s*\#!\s*/) {
-        return 1;
-    } else {
-        return 0;
-    }
-}
-
 =head2 _find_all_files()
 
 This is used by process_app_files() to get the list of files under "extra_dirs"
@@ -519,41 +433,34 @@ to copy to "blib".
 sub _find_all_files {
     my ($self, $dir) = @_;
     return {} unless -d $dir;
-    return { map { $_, $_ } @{ $self->rscan_dir($dir, sub { -f $_ }) } };
-}
-
-=head2 rscan_dir()
-
-Don't include CVS, RCS, and SVN (*/.svn/*) files.
-
-=cut
-
-sub rscan_dir {
-    my ($self, $dir, $pattern) = @_;
-    my $files = $self->SUPER::rscan_dir($dir, $pattern);
-    my @files = grep(!/[\/\\](CVS|RCS|\.svn)[\/\\]/, @$files);
-    return \@files;
+    return { map { $_, $_ } @{ $self->rscan_dir($dir, sub { $_ !~ /(CVS|RCS|SCCS)/ }) } };
 }
 
 ######################################################################
 # INSTALL: enhancements to "./Build install"
 ######################################################################
 
-=head2 packlist()
+=head2 install_base_relative()
 
-This creates the name of the "packlist" file that needs to be
-written with the list of all of the files that get installed.
+This method is overridden to indicate that the relative directory
+of an "extra_dir" is the same as the dir/type itself.
 
 =cut
 
-sub packlist {
-    my ($self) = @_;
-    # Write the packlist into the same place as ExtUtils::MakeMaker.
-    my $archdir = $self->install_destination('arch');
-    my @ext = $self->module_name ? split /::/, $self->module_name :
-                                   $self->dist_name;
-    my $packlist = File::Spec->catfile($archdir, 'auto', @ext, '.packlist');
-    return($packlist);
+sub install_base_relative {
+    my ($self, $type) = @_;
+    my $extra_dirs = $self->_get_extra_dirs_attributes();
+    my ($reldir);
+    if ($extra_dirs->{$type}) {
+        $reldir = $type;
+    }
+    elsif ($type eq "html") {
+        $reldir = "htdocs/docs";
+    }
+    if (!$reldir) {
+        $reldir = $self->SUPER::install_base_relative($type);
+    }
+    return($reldir);
 }
 
 =head2 install_map()
@@ -574,12 +481,6 @@ sub install_map {
     next unless -e $localdir;
 
     if (my $dest = $self->install_destination($type)) {
-      # thins alters the behavious of Module::Build, and
-      # looks into the implementation
-      if (   $self->install_path($type)
-          && !File::Spec->file_name_is_absolute($dest)) {
-        $dest = File::Spec->catdir( $self->_prefix, $dest );
-      }
       $map{$localdir} = $dest;
     } else {
       # Platforms like Win32, MacOS, etc. may not build man pages
@@ -588,17 +489,14 @@ sub install_map {
     }
   }
 
-  my $extra_dirs_attrs = $self->_get_extra_dirs_attributes();
-  foreach my $dir ( $self->_get_extra_dirs() ) {
-    my $dest = $extra_dirs_attrs->{$dir}{dest_dir};
-    if (!File::Spec->file_name_is_absolute($dest)) {
-      $dest = File::Spec->catdir( $self->_prefix, $dest );
-    }
-    $map{File::Spec->catdir( $blib, $dir )} = $dest;
-  }
-
   if ($self->create_packlist) {
-    $map{write} = $self->packlist();
+    # Write the packlist into the same place as ExtUtils::MakeMaker.
+    my $archdir = $self->install_destination('arch');
+    my @ext = split /::/, $self->module_name;
+    if ($#ext == -1) {              # FIX
+        @ext = ($self->dist_name);  # FIX
+    }                               # FIX
+    $map{write} = File::Spec->catfile($archdir, 'auto', @ext, '.packlist'); # FIX ?!?
   }
 
   if (length(my $destdir = $self->{properties}{destdir} || '')) {
@@ -615,18 +513,6 @@ sub install_map {
   return \%map;
 }
 
-=head2 has_config_data()
-
-No. We're not using config data.
-Always return FALSE.
-
-=cut
-
-sub has_config_data {
-    my ($self) = @_;
-    return(0);
-}
-
 =head2 ACTION_install()
 
 This method is overridden to put in the configure() hook so that
@@ -642,24 +528,9 @@ sub ACTION_install {
     $self->depends_on('build');
     my $map = $self->install_map;
     ExtUtils::Install::install($map, 1, 0, $self->{args}{uninst}||0);
-    $self->perllocal_install();
     $self->configure();
 }
 
-=head2 perllocal_install()
-
-This method should be modelled after ExtUtils::Command::MM::perllocal_install
-so that it writes the same information at MakeMaker does.
-
-It currently is a stub, waiting to be implemented
-
-=cut
-
-sub perllocal_install {
-    my ($self) = @_;
-    # Not yet implemented.
-}
-
 =head2 configure()
 
 Do nothing.  This method is a hook that can be overridden by a 
@@ -674,81 +545,10 @@ sub configure {
     # Do nothing.  This is a hook for overriding in a subclass.
 }
 
-=head2 mirror()
-
-    * Signature: $build->mirror($url, $file);
-    * Param:  $url          string
-    * Param:  $file         string
-
-TODO: Should be rewritten to use cross-platform, pure-perl.
-
-=cut
-
-sub mirror {
-    my ($self, $url, $file) = @_;
-    if (! -f $file) {
-        $self->log_info("Mirroring $url to $file\n");
-        require File::Fetch;
-        my $ff = File::Fetch->new(uri => $url);
-        my $where = $ff->fetch(to => File::Basename::dirname($file));
-        if($where) {
-            rename($where, $file);
-        }
-    }
-    else {
-        $self->log_info("Mirrored file $file up to date\n");
-    }
-}
-
-=head2 unpack()
-
-    * Signature: $build->unpack($archive_file, $directory, $subdir);
-    * Param:  $archive_file string
-    * Param:  $directory    string
-    * Param:  $subdir       string
-
-TODO: Should be rewritten to use cross-platform, pure-perl.
-
-=cut
-
-sub unpack {
-    my ($self, $archive_file, $directory, $subdir) = @_;
-    require Archive::Extract;
-    $directory ||= "$App::options{install_prefix}/src";
-    mkdir($directory) if (! -d $directory);
-    die "Directory $directory does not exist and can't be created" if (! -d $directory);
-
-    if (! File::Spec->file_name_is_absolute($archive_file)) {
-        $archive_file = File::Spec->catfile(Cwd::getcwd(), $archive_file);
-    }
-    $subdir = File::Spec->catdir($directory, $subdir);
-
-    if ($subdir && -d $subdir) {
-        $self->log_info("Removing preexisting directory $subdir ...\n");
-        File::Path::rmtree($subdir);
-    }
-    $self->log_info("Unpacking $archive_file ...\n");
-
-    my $ae = Archive::Extract->new(archive => $archive_file);
-    my $ok = $ae->extract(to => $directory) or die $ae->error;
-
-    die "Subdirectory $subdir not created" if (! -d $subdir);
-}
-
 =head1 ACKNOWLEDGEMENTS
 
  * Author:  Stephen Adkins <stephen.adkins@officevision.com>
- * Maintainer: Mattia Barbon <mbarbon@cpan.org>
-
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
-=head1 SOURCES
-
-The latest sources can be found on GitHub at
-L<http://github.com/mbarbon/app-build/tree>
+ * License: This is free software. It is licensed under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
@@ -758,3 +558,79 @@ L<http://github.com/mbarbon/app-build/tree>
 
 __END__
 
+# NO LONGER USED: using Sysadm::Install instead
+
+#=head2 mirror()
+
+    * Signature: App::Build->mirror($url, $file);
+    * Param:  $url          string
+    * Param:  $file         string
+    * Since:  0.50
+
+#=cut
+
+sub mirror {
+    my ($class, $url, $file) = @_;
+    if (! -f $file) {
+        print "Mirroring $url to $file\n";
+        system("wget -O $file $url");
+    }
+    else {
+        print "Mirrored file $file up to date\n";
+    }
+}
+
+#=head2 unpack()
+
+    * Signature: App::Build->unpack($archive_file, $directory, $subdir);
+    * Param:  $archive_file string
+    * Param:  $directory    string
+    * Param:  $subdir       string
+    * Since:  0.50
+
+#=cut
+
+sub unpack {
+    my ($class, $archive_file, $directory, $subdir) = @_;
+    my $verbose = $App::options{verbose};
+    $directory ||= "$App::options{install_prefix}/src";
+    mkdir($directory) if (! -d $directory);
+    die "Directory $directory does not exist and can't be created" if (! -d $directory);
+
+    my $start_dir = Cwd::getcwd();
+    if (! File::Spec->file_name_is_absolute($archive_file)) {
+        $archive_file = File::Spec->catfile($start_dir, $archive_file);
+    }
+
+    chdir($directory);
+    if ($subdir && -d $subdir) {
+        print "Removing preexisting directory $subdir ...\n" if ($verbose);
+        system("rm -rf $subdir");
+        print "Removing done.\n" if ($verbose);
+    }
+    print "Unpacking $archive_file ...\n" if ($verbose);
+
+    # my $archive = Archive::Any->new($archive_file);
+    # $archive->extract;
+    # my @files = $archive->files;
+    # my $type = $archive->type;
+    # $archive->is_impolite;
+    # $archive->is_naughty;
+
+    if ($archive_file =~ /\.zip$/) {
+        system("unzip $archive_file");
+    }
+    elsif ($archive_file =~ /\.tar\.gz$/ || $archive_file =~ /\.tgz$/) {
+        system("tar xvzf $archive_file");
+    }
+    else {
+        die "Unknown archive type: $archive_file\n";
+    }
+
+    print "Unpacking done.\n" if ($verbose);
+
+    die "Subdirectory $subdir not created" if (! -d $subdir);
+
+    chdir($start_dir);
+}
+
